import {
  AddCdnDomainRequest,
  Origin,
  ServerCert,
} from 'tencentcloud-sdk-nodejs/tencentcloud/services/cdn/v20180606/cdn_models';
import { pascalCaseProperty, removeUndefined } from '@tencent-serverless-framework/common';
import { CdnDeployInputs, CertInputs, OriginInputs } from './interface';
import * as fs from 'fs';
import * as path from 'path';

/** 用于获取证书文件信息 */
function getCertPathContent(target: string) {
  let content = '';

  try {
    const stat = fs.statSync(target);
    if (stat.isFile()) {
      if (path.isAbsolute(target)) {
        content = fs.readFileSync(target, 'base64');
      } else {
        content = fs.readFileSync(path.join(process.cwd(), target), 'base64');
      }
    }
  } catch (e) {
    // target is string just return
    content = target;
  }
  return content;
}

/**
 * 格式化证书
 * @param {object} inputs cert info
 */
function certInputs(inputs: CertInputs): ServerCert {
  if (inputs.certId) {
    return {
      CertId: inputs.certId,
    };
  }
  return {
    Certificate: getCertPathContent(inputs.certificate!),
    PrivateKey: getCertPathContent(inputs.privateKey!),
    Message: inputs.remarks ?? '',
  };
}

/** 格式化 CDN 源信息 */
export function originInputs(inputs: OriginInputs): Origin {
  const originInfo: Origin = {
    Origins: inputs.origins,
    OriginType: inputs.originType,
    OriginPullProtocol: inputs.originPullProtocol,
    ServerName: inputs.serverName,
  };
  if (inputs.originType === 'cos') {
    return {
      ...originInfo,
      ServerName: inputs.origins![0] ?? '',
      CosPrivateAccess: 'off',
    };
  }
  if (inputs.originType === 'domain') {
    if (inputs.backupOrigins) {
      return {
        ...originInfo,
        BackupOrigins: inputs.backupOrigins,
        BackupOriginType: inputs.backupOriginType,
        BackupServerName: inputs.backupServerName,
      };
    }
  }
  return originInfo;
}

/** DeployInputs 转 AddCdnDomainRequest */
function deployInputs(inputs: CdnDeployInputs): AddCdnDomainRequest {
  const cdnInputs = pascalCaseProperty(inputs);
  if (cdnInputs.Https) {
    const hs = inputs.https;
    cdnInputs.Https = {
      Switch: hs?.switch ?? 'on',
      Http2: hs?.http2 ?? 'off',
      OcspStapling: hs?.ocspStapling ?? 'off',
      VerifyClient: hs?.verifyClient ?? 'off',
      CertInfo: certInputs(hs?.certInfo!),
    };
  }
  if (inputs.forceRedirect && inputs.https) {
    cdnInputs.ForceRedirect = {
      Switch: inputs.forceRedirect.switch ?? 'on',
      RedirectStatusCode: inputs.forceRedirect.redirectStatusCode ?? 301,
      RedirectType: 'https',
    };
  }
  cdnInputs.Origin = originInputs(inputs.origin);

  /** 移除 CDN 参数中多余的参数，否则会导致部署失败 */

  cdnInputs.OldState = undefined;
  cdnInputs.Async = undefined;
  cdnInputs.OnlyRefresh = undefined;
  cdnInputs.RefreshCdn = undefined;
  cdnInputs.PushCdn = undefined;

  return removeUndefined(cdnInputs);
}

export const formatter = {
  originInputs,
  certInputs,
  deployInputs,
};
